% fn_clean_bcei_lr.m
% Function to clean Blue Chip Economic Indicators long-run inflation
% estimates.  
%  [bcei_lr_pce,bcei_lr_cpi]=fn_clean_bcei_lr(bcei_lr_raw,date_q,linearly_interpolate);
%
% The function loads the raw Blue Chip long-run inflation forecasts and
% adds junk numbers where necessary.  The function can either leave
% the raw data as-is and fill in missing observations with the most 
% recent entry, or it can linearly interpolate between entries
% to produce a quarterly series.
%
% Inputs:
% bcei_lr_raw - A matrix with the raw data
% date_q - A vector with dates for the main program
% linearly_interpolate - A dummy, 1 means linearly interpolate to fill
%                        in missing observations, 0 means not to
%
% Outputs:
% bcei_lr_pce - A vector with PCE estimates, assumed to be equal to GDP deflator estimates
% bcei_lr_cpi - A vector with CPI estimates
%
function [bcei_lr_pce,bcei_lr_cpi]=fn_clean_bcei_lr(bcei_lr_raw,date_q,linearly_interpolate);
%
global junk_nos;
% data
bcei_lr_dates=bcei_lr_raw(:,1); % dates in column 1
bcei_lr_pce_raw=bcei_lr_raw(:,2); % GDP deflator, assumed to be same as PCE, in column 2
bcei_lr_cpi_raw=bcei_lr_raw(:,3); % CPI in column 3
% set up outputs
bcei_lr_pce=junk_nos*ones(size(date_q));
bcei_lr_cpi=junk_nos*ones(size(date_q));
% copy over entries as they stand
for i1=1:length(bcei_lr_dates);
	i2=find(date_q==bcei_lr_dates(i1));
	bcei_lr_pce(i2)=bcei_lr_pce_raw(i1);
	bcei_lr_cpi(i2)=bcei_lr_cpi_raw(i1);
end;
% fill in missing entries
if linearly_interpolate==1; % linearly interpolate, except for any entries after the last observation, assume a random walk
	pos1=find(bcei_lr_pce~=junk_nos);
	for i2=2:length(pos1);
		if pos1(i2)~=pos1(i2-1)+1;
			pos_low=pos1(i2-1);
			pos_high=pos1(i2);
			m=(bcei_lr_pce(pos_low)-bcei_lr_pce(pos_high))/(date_q(pos_low)-date_q(pos_high));
			b=bcei_lr_pce(pos_low)-m*date_q(pos_low);
			for i3=pos_low+1:pos_high-1;
				bcei_lr_pce(i3)=m*date_q(i3)+b;
			end;
		end;
	end;
	bcei_lr_pce(pos1(end)+1:end)=bcei_lr_pce(pos1(end));
	pos1=find(bcei_lr_cpi~=junk_nos);
	for i2=2:length(pos1);
		if pos1(i2)~=pos1(i2-1)+1;
			pos_low=pos1(i2-1);
			pos_high=pos1(i2);
			m=(bcei_lr_cpi(pos_low)-bcei_lr_cpi(pos_high))/(date_q(pos_low)-date_q(pos_high));
			b=bcei_lr_cpi(pos_low)-m*date_q(pos_low);
			for i3=pos_low+1:pos_high-1;
				bcei_lr_cpi(i3)=m*date_q(i3)+b;
			end;
		end;
	end;
	bcei_lr_cpi(pos1(end)+1:end)=bcei_lr_cpi(pos1(end));
else; % just carry forward the previous reading
	pos1=find(bcei_lr_pce~=junk_nos);
	for i2=2:length(pos1);
		if pos1(i2)~=pos1(i2-1)+1;
			pos_low=pos1(i2-1);
			pos_high=pos1(i2);
			bcei_lr_pce(pos_low+1:pos_high-1)=bcei_lr_pce(pos_low);
		end;
	end;
	bcei_lr_pce(pos1(end)+1:end)=bcei_lr_pce(pos1(end));
	pos1=find(bcei_lr_cpi~=junk_nos);
	for i2=2:length(pos1);
		if pos1(i2)~=pos1(i2-1)+1;
			pos_low=pos1(i2-1);
			pos_high=pos1(i2);
			bcei_lr_cpi(pos_low+1:pos_high-1)=bcei_lr_cpi(pos_low);
		end;
	end;
	bcei_lr_cpi(pos1(end)+1:end)=bcei_lr_cpi(pos1(end));
end;