% fn_make_forecast_alternatives_q.m
% Function to provide alternative forecasts for quarterly nowcasts.
%  [g_p_q_nowcasts_alt,g_p_q_nowcasts_alt_last]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,p,g_p_q_lr);
%
% Considers alternative cases to compare against the baseline model.
% 1. Inflation over the last four available quarters
% 2. AR(1) in gaps forecast (whole sample of Blue Chip data)
% 3. UCSV (Stock and Watson 2007) forecast (starting 1960)
%
% Inputs:
% date_q - A vector with dates in yyyy.q format.
% qcount - The current quarter position of interest.
% date_m - A vector with dates in yyyy.mm format.
% p - A vector with the available data.
% g_p_q_lr - A vector with long-run forecasts of p, frequency is quarterly.
%
% Outputs:
% g_p_q_nowcasts_alt - A vector with the alternative forecasts.
% g_p_q_nowcasts_alt_last - A vector with the previous period's data/forecast coming from each alternative forecast.
%
function [g_p_q_nowcasts_alt,g_p_q_nowcasts_alt_last]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,p,g_p_q_lr);
%
global junk_nos;
%
no_alt_cases=3;
g_p_q_nowcasts_alt=junk_nos*ones(no_alt_cases,1);
g_p_q_nowcasts_alt_last=junk_nos*ones(no_alt_cases,1);
% convert what is available to quarterly frequency
[p_q,g_p_q]=fn_mtoqavg(p,date_m,date_q);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Inflation over the last four available quarters
g_p_q_nowcasts_alt(1)=100*(p_q(i1)/p_q(i1-4)-1);
if i1<qcount-1;
	g_p_q_nowcasts_alt_last(1)=100*(p_q(i1)/p_q(i1-4)-1);
else;
	g_p_q_nowcasts_alt_last(1)=g_p_q(qcount-1);
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. AR(1) in gaps forecast (whole sample of Blue Chip data)
f_g_p_q=g_p_q;
i1=max(find(g_p_q~=junk_nos));
f_g_p_q_lr=junk_nos*ones(size(f_g_p_q));
f_g_p_q_lr(1:i1)=g_p_q_lr(1:i1);
i2=max(min(find(g_p_q~=junk_nos)),min(find(f_g_p_q_lr~=junk_nos)));
f_g_gap_q=junk_nos*ones(size(f_g_p_q));
x=g_p_q(i2:i1)-g_p_q_lr(i2:i1);
f_g_gap_q(i2:i1)=x;
t=length(x)-1;
b=regress(x(2:end),[ones(t,1),x(1:end-1)]);
for i2=i1+1:qcount;
	f_g_gap_q(i2)=b'*[1;f_g_gap_q(i2-1)];
	f_g_p_q_lr(i2)=f_g_p_q_lr(i2-1);
	f_g_p_q(i2)=f_g_gap_q(i2)+f_g_p_q_lr(i2);
end;
g_p_q_nowcasts_alt(2)=f_g_p_q(qcount);
g_p_q_nowcasts_alt_last(2)=f_g_p_q(qcount-1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. UCSV from Stock and Watson forecast (starting 1960)
f_g_p_q=g_p_q;
i1=max(find(g_p_q~=junk_nos));
i2=max(find(date_q==1960.1),min(find(g_p_q~=junk_nos)));
x=g_p_q(i2:i1);
if i1+1<=qcount;
	f_g_p_q(i1+1:qcount)=fn_ucsv_stock_watson(x);
end;
g_p_q_nowcasts_alt(3)=f_g_p_q(qcount);
g_p_q_nowcasts_alt_last(3)=f_g_p_q(qcount-1);