% fn_pesaran_timmermann_sc_dfa.m
% Function to compute Pesaran and Timmermann (2009) serially correlated multicategory dependence;
% applicable to directional forecast accuracy.
%  pt=fn_pesaran_timmermann_sc_dfa(x,y);
%
% Note: Code is written under the assumption that there are two categories, increase and decrease/no change,
% but the procedure is more general.
%
% Inputs:
% x - For forecasts, a vector =1 if the forecast increased and 0 otherwise
% y - For the data, a vector =1 if the data increased and 0 otherwise
%
% Output: 
% pt - the test statistic, ~ chi-squared(1) (if there are only two categories)
% pvalue - the test statistic's p value, assume only two categories
% success_ratio - the success ratio, or the percentage of the time that forecasted direction = actual direction
%
function [pt,pvalue,success_ratio]=fn_pesaran_timmermann_sc_dfa(x,y);
%
% If all observations are successes:
if sum(x==y)==length(x);
	pt=NaN;
	pvalue=NaN;
	success_ratio=1;
	return;
end;
% If there is no variation in x (always predicted an increase or a non-increase)
if sum(x==0)==length(x) | sum(x==1)==length(x);
	pt=NaN;
	pvalue=NaN;
	success_ratio=sum(x==y)/length(y);
	return;
end;
% Number of observations
T=size(y,1);
x0=x(1:T-1);
x1=x(2:T);
y0=y(1:T-1);
y1=y(2:T);
tau=ones(T-1,1);
W=[tau,x0,y0];
Mw=eye(T-1)-W*inv(W'*W)*W';
Syyw=y1'*Mw*y1/(T-1);
Sxxw=x1'*Mw*x1/(T-1);
Sxyw=x1'*Mw*y1/(T-1);
Syxw=y1'*Mw*x1/(T-1);
% Statistic:
pt=inv(Syyw)*Syxw*inv(Sxxw)*Sxyw*(T-1);
% Chi-squared p-value, assuming 2 categories
pvalue=1-chi2cdf(pt,1);
% Success ratio
success_ratio=sum(x==y)/T;